from manga_py.provider import Provider
from .helpers.std import Std


class JaiminisBoxNet(Provider, Std):
    _manga_idx = None
    _re = None

    def get_chapter_index(self) -> str:
        return self._re.search(self.chapter).group(1)

    def get_content(self):
        content = self._get_content('{}/manga/{}')
        self.__set_idx(content)
        return content

    def get_manga_name(self) -> str:
        return self._get_name(r'/manga/([^/]+)/')

    def get_chapters(self):
        if self._manga_idx is None:
            return []

        content = self.http_post(
            'https://jaiminisbox.net/wp-admin/admin-ajax.php',
            data={
                'action': 'manga_get_chapters',
                'manga': self._manga_idx,
            }
        )

        return self._elements(
            '.listing-chapters_wrap .wp-manga-chapter > a',
            content
        )

    def get_files(self):
        return self._images_helper(
            self.html_fromstring(self.chapter),
            '.reading-content img.wp-manga-chapter-img',
        )

    def get_cover(self) -> str:
        return self._cover_from_content('.summary_image > a > img')

    def prepare_cookies(self):
        self._re = self.re.compile(r'/chapter-(\d+)/')

    def __set_idx(self, content):
        parser = self.document_fromstring(content, 'input.rating-post-id', 0)
        if parser is not None:
            self._manga_idx = int(parser.get('value'))


main = JaiminisBoxNet



import time
from abc import abstractmethod
from logging import info


class Abstract:

    @abstractmethod
    def get_content(self):  # call once
        pass

    @abstractmethod
    def get_manga_name(self) -> str:  # call once
        return ''

    @abstractmethod
    def get_chapters(self) -> list:  # call once
        return []

    def prepare_cookies(self):  # if site with cookie protect
        pass

    @abstractmethod
    def get_files(self) -> list:  # call ever volume loop
        return []

    # @abstractmethod
    # def get_archive_name(self) -> str:
    #     pass

    #  for chapters selected by manual (cli)
    @abstractmethod
    def get_chapter_index(self) -> str:
        pass

    def book_meta(self) -> dict:
        pass

    def before_download_chapter(self):
        pass

    def after_download_chapter(self):
        pass

    def get_cover(self):
        pass

    def before_file_save(self, url, idx) -> str:  # return url !
        return url

    def after_file_save(self, _path: str, idx: int):
        self._wait_after_file()
        return _path, None

    @abstractmethod
    def chapter_for_json(self) -> str:
        pass

    def _wait_after_file(self):
        if hasattr(self, '_params'):
            wait_time = self._params.get('wait_after_page', 0)

            if wait_time > 0:
                info(f'Wait {wait_time} second(s) / file')
                time.sleep(wait_time)

    def _wait_after_chapter(self):
        if hasattr(self, '_params'):
            wait_time = self._params.get('wait_after_chapter', 0)

            if wait_time > 0:
                info(f'Wait {wait_time} second(s) / chapter')
                time.sleep(wait_time)



from .manganelo_com import MangaNeloCom


class MangaNatoCom(MangaNeloCom):
    _prefix = '/manga-'
    __alternative_cdn = 'https://bu2.mkklcdnv6tempv2.com'

    def get_manga_name(self) -> str:
        return self._get_name('/manga-([^/]+)')

    def prepare_cookies(self):
        self.http().allow_send_referer = True
        self.http().referer = self.domain


main = MangaNatoCom



from manga_py.provider import Provider
from .helpers.std import Std


class LittleXGardenCom(Provider, Std):
    __images_webroot = 'https://littlexgarden.com/static/images/'

    def get_chapter_index(self) -> str:
        return self.re.search(r'\.\w{2,5}/[^/]+/(\d+)', self.chapter).group(1)

    def get_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,5}/([^/]+)')

    def get_chapters(self):
        return self._elements('.chapters > .list a[title]')[::-1]

    def get_files(self):
        content = self.http_get(self.chapter)
        script_content = None

        scripts = self.document_fromstring(content, 'script')

        nuxt_re = self.re.compile(r'__NUXT__\s?=')
        image_re = self.re.compile(r'"([a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}\.jpg)"')

        for script in scripts:
            source = self.element_text_content(script)
            if nuxt_re.search(source):
                script_content = source
                break

        if script_content is None:
            self.log('Images not found')
            return []

        return list(map(self._image_url, image_re.findall(script_content)))

    def _image_url(self, image_id: str):
        if self.http().allow_webp:
            return f'{self.__images_webroot}webp/{image_id}.webp'
        return f'{self.__images_webroot}{image_id}'


main = LittleXGardenCom
